import {
  RouteRequest,
  RouteResponse,
} from '@modules/ad/core/application/ports/georouter.port';
import {
  RouteCompleter,
  RouteCompleterType,
} from '@modules/ad/core/application/queries/match/completer/route.completer';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { Actor } from '@modules/ad/core/domain/value-objects/actor.value-object';
import { Step } from '@modules/geography/core/domain/route.types';
import { simpleMockGeorouter } from '../georouter.mock';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery(
  {
    algorithmType: AlgorithmType.PASSENGER_ORIENTED,
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-08-28',
    toDate: '2023-08-28',
    schedule: [
      {
        time: '07:05',
      },
    ],
    strict: false,
    waypoints: [originWaypoint, destinationWaypoint],
  },
  {
    getRoute: jest
      .fn()
      .mockImplementation(async (req: RouteRequest): Promise<RouteResponse> => {
        const response = await simpleMockGeorouter.getRoute(req);
        if (req.detailsSettings?.steps) {
          const step: Step = { lon: 0, lat: 0, duration: 0 };
          response.steps = [step, step, step, step];
        }
        return response;
      }),
  },
);

const candidate: CandidateEntity = CandidateEntity.create({
  id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
  role: Role.DRIVER,
  frequency: Frequency.PUNCTUAL,
  dateInterval: {
    lowerDate: '2023-08-28',
    higherDate: '2023-08-28',
  },
  driverWaypoints: [
    {
      lat: 48.678454,
      lon: 6.189745,
    },
    {
      lat: 48.84877,
      lon: 2.398457,
    },
  ],
  passengerWaypoints: [
    {
      lat: 48.689445,
      lon: 6.17651,
    },
    {
      lat: 48.8566,
      lon: 2.3522,
    },
  ],
  driverDistance: 350145,
  driverDuration: 13548,
  driverSchedule: [
    {
      day: 0,
      time: '07:00',
      margin: 900,
    },
  ],
  passengerSchedule: [
    {
      day: 0,
      time: '07:10',
      margin: 900,
    },
  ],
  spacetimeDetourRatio: {
    maxDistanceDetourRatio: 0.3,
    maxDurationDetourRatio: 0.3,
  },
}).setCarpoolPath([
  {
    lat: 48.689445,
    lon: 6.17651,
    actors: [
      new Actor({
        role: Role.DRIVER,
        target: Target.START,
      }),
      new Actor({
        role: Role.PASSENGER,
        target: Target.START,
      }),
    ],
  },
  {
    lat: 48.8566,
    lon: 2.3522,
    actors: [
      new Actor({
        role: Role.DRIVER,
        target: Target.FINISH,
      }),
      new Actor({
        role: Role.PASSENGER,
        target: Target.FINISH,
      }),
    ],
  },
]);

describe('Route completer', () => {
  it('should complete candidates with basic setting', async () => {
    const routeCompleter: RouteCompleter = new RouteCompleter(
      matchQuery,
      RouteCompleterType.BASIC,
    );
    const completedCandidates: CandidateEntity[] =
      await routeCompleter.complete([candidate]);
    expect(completedCandidates.length).toBe(1);
    expect(completedCandidates[0].getProps().distance).toBe(350101);
  });
  it('should complete candidates with detailed setting', async () => {
    const routeCompleter: RouteCompleter = new RouteCompleter(
      matchQuery,
      RouteCompleterType.DETAILED,
    );
    const completedCandidates: CandidateEntity[] =
      await routeCompleter.complete([candidate]);
    expect(completedCandidates.length).toBe(1);
    expect(completedCandidates[0].getProps().steps).toHaveLength(4);
  });
});
